@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2008 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,12 @@
 #include <time.h>
 #include <gdk/gdk.h>
 
+#include <gconf/gconf-engine.h>
+#include <gconf/gconf-client.h>
+
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libgnomeui/gnome-bg.h>
+
 #include "gs-prefs.h"        /* for GSSaverMode */
 
 #include "gs-manager.h"
@@ -47,6 +53,9 @@ struct GSManagerPrivate
         GHashTable  *jobs;
 
         GSThemeManager *theme_manager;
+        GConfClient    *client;
+        GnomeBG        *bg;
+        guint           bg_notify_id;
 
         /* Policy */
         glong        lock_timeout;
@@ -932,6 +941,69 @@ gs_manager_class_init (GSManagerClass *klass)
         g_type_class_add_private (klass, sizeof (GSManagerPrivate));
 }
 
+static void
+on_bg_changed (GnomeBG   *bg,
+               GSManager *manager)
+{
+        gs_debug ("background changed");
+}
+
+static void
+gconf_changed_callback (GConfClient *client,
+                        guint        cnxn_id,
+                        GConfEntry  *entry,
+                        GSManager   *manager)
+{
+        gnome_bg_load_from_preferences (manager->priv->bg,
+                                        manager->priv->client);
+}
+
+static void
+watch_bg_preferences (GSManager *manager)
+{
+        g_assert (manager->priv->bg_notify_id == 0);
+
+        gconf_client_add_dir (manager->priv->client,
+                              GNOME_BG_KEY_DIR,
+                              GCONF_CLIENT_PRELOAD_NONE,
+                              NULL);
+        manager->priv->bg_notify_id = gconf_client_notify_add (manager->priv->client,
+                                                               GNOME_BG_KEY_DIR,
+                                                               (GConfClientNotifyFunc)gconf_changed_callback,
+                                                               manager,
+                                                               NULL,
+                                                               NULL);
+}
+
+static GConfClient *
+get_gconf_client (void)
+{
+        GConfClient        *client;
+        GSList             *addresses;
+        GError             *error;
+        GConfEngine        *engine;
+
+        client = NULL;
+        addresses = NULL;
+
+        addresses = g_slist_prepend (addresses, "xml:merged:" SYSCONFDIR "/gconf/gconf.xml.mandatory");
+        addresses = g_slist_prepend (addresses, "xml:merged:" SYSCONFDIR "/gconf/gconf.xml.system");
+        addresses = g_slist_prepend (addresses, "xml:merged:" SYSCONFDIR "/gconf/gconf.xml.defaults");
+        addresses = g_slist_reverse (addresses);
+
+        error = NULL;
+        engine = gconf_engine_get_for_addresses (addresses, &error);
+        if (engine == NULL) {
+                gs_debug ("Unable to get gconf engine for addresses: %s", error->message);
+        } else {
+                client = gconf_client_get_for_engine (engine);
+        }
+
+        g_slist_free (addresses);
+
+        return client;
+}
+
 static void
 gs_manager_init (GSManager *manager)
 {
@@ -940,6 +1012,19 @@ gs_manager_init (GSManager *manager)
         manager->priv->fade = gs_fade_new ();
         manager->priv->grab = gs_grab_new ();
         manager->priv->theme_manager = gs_theme_manager_new ();
+
+        manager->priv->client = get_gconf_client ();
+        if (manager->priv->client != NULL) {
+                manager->priv->bg = gnome_bg_new ();
+
+                g_signal_connect (manager->priv->bg,
+                                  "changed",
+                                  G_CALLBACK (on_bg_changed),
+                                  manager);
+                watch_bg_preferences (manager);
+
+                gnome_bg_load_from_preferences (manager->priv->bg, manager->priv->client);
+        }
 }
 
 static void
@@ -958,11 +1043,52 @@ remove_unfade_idle (GSManager *manager)
         }
 }
 
+
+static void
+on_screen_monitors_changed (GdkScreen *screen,
+                            GSManager *manager)
+{
+        gs_debug ("Monitors changed for screen %d: num=%d",
+                  gdk_screen_get_number (screen),
+                  gdk_screen_get_n_monitors (screen));
+}
+
+static void
+gs_manager_destroy_windows (GSManager *manager)
+{
+        GdkDisplay  *display;
+        GSList      *l;
+        int          n_screens;
+        int          i;
+
+        g_return_if_fail (manager != NULL);
+        g_return_if_fail (GS_IS_MANAGER (manager));
+
+        if (manager->priv->windows == NULL) {
+                return;
+        }
+
+        display = gdk_display_get_default ();
+
+        n_screens = gdk_display_get_n_screens (display);
+
+        for (i = 0; i < n_screens; i++) {
+                g_signal_handlers_disconnect_by_func (gdk_display_get_screen (display, i),
+                                                      on_screen_monitors_changed,
+                                                      manager);
+        }
+
+        for (l = manager->priv->windows; l; l = l->next) {
+                gs_window_destroy (l->data);
+        }
+        g_slist_free (manager->priv->windows);
+        manager->priv->windows = NULL;
+}
+
 static void
 gs_manager_finalize (GObject *object)
 {
         GSManager *manager;
-        GSList    *l;
 
         g_return_if_fail (object != NULL);
         g_return_if_fail (GS_IS_MANAGER (object));
@@ -971,6 +1097,21 @@ gs_manager_finalize (GObject *object)
 
         g_return_if_fail (manager->priv != NULL);
 
+        if (manager->priv->bg_notify_id != 0) {
+                gconf_client_remove_dir (manager->priv->client,
+                                         GNOME_BG_KEY_DIR,
+                                         NULL);
+                gconf_client_notify_remove (manager->priv->client,
+                                            manager->priv->bg_notify_id);
+                manager->priv->bg_notify_id = 0;
+        }
+        if (manager->priv->bg != NULL) {
+                g_object_unref (manager->priv->bg);
+        }
+        if (manager->priv->client != NULL) {
+                g_object_unref (manager->priv->client);
+        }
+
         free_themes (manager);
         g_free (manager->priv->logout_command);
         g_free (manager->priv->keyboard_command);
@@ -983,11 +1124,7 @@ gs_manager_finalize (GObject *object)
 
         manager_stop_jobs (manager);
 
-        for (l = manager->priv->windows; l; l = l->next) {
-                gs_window_destroy (l->data);
-        }
-        g_slist_free (manager->priv->windows);
-        manager->priv->windows = NULL;
+        gs_manager_destroy_windows (manager);
 
         manager->priv->active = FALSE;
         manager->priv->activate_time = 0;
@@ -1216,13 +1353,42 @@ window_unmap_cb (GSWindow  *window,
         gs_debug ("window unmapped!");
 }
 
+static void
+apply_background_to_window (GSManager *manager,
+                            GSWindow  *window)
+{
+        GdkPixmap       *pixmap;
+        GdkScreen       *screen;
+        int              width;
+        int              height;
+
+        if (manager->priv->bg == NULL) {
+                gs_debug ("No background available");
+                gs_window_set_background_pixmap (window, NULL);
+        }
+
+        screen = gs_window_get_screen (window);
+        width = gdk_screen_get_width (screen);
+        height = gdk_screen_get_height (screen);
+        gs_debug ("Creating pixmap background w:%d h:%d", width, height);
+        pixmap = gnome_bg_create_pixmap (manager->priv->bg,
+                                         gs_window_get_gdk_window (window),
+                                         width,
+                                         height,
+                                         TRUE);
+        gs_window_set_background_pixmap (window, pixmap);
+        g_object_unref (pixmap);
+}
+
 static void
 manager_show_window (GSManager *manager,
                      GSWindow  *window)
 {
         GSJob *job;
 
-        job = gs_job_new_for_widget (GTK_WIDGET (window));
+        apply_background_to_window (manager, window);
+
+        job = gs_job_new_for_widget (gs_window_get_drawing_area (window));
 
         manager_select_theme_for_job (manager, job);
         manager_add_job_for_window (manager, window, job);
@@ -1351,8 +1517,8 @@ connect_window_signals (GSManager *manager,
 }
 
 static void
-gs_manager_create_window (GSManager *manager,
-                          GdkScreen *screen)
+gs_manager_create_windows_for_screen (GSManager *manager,
+                                      GdkScreen *screen)
 {
         GSWindow *window;
         int       n_monitors;
@@ -1390,26 +1556,27 @@ gs_manager_create_window (GSManager *manager,
 }
 
 static void
-gs_manager_create (GSManager *manager)
+gs_manager_create_windows (GSManager *manager)
 {
         GdkDisplay  *display;
-        GSList      *l;
-        int          n_screens, i;
+        int          n_screens;
+        int          i;
 
         g_return_if_fail (manager != NULL);
         g_return_if_fail (GS_IS_MANAGER (manager));
 
-        display = gdk_display_get_default ();
+        g_assert (manager->priv->windows == NULL);
 
+        display = gdk_display_get_default ();
         n_screens = gdk_display_get_n_screens (display);
 
-        for (l = manager->priv->windows; l; l = l->next) {
-                gs_window_destroy (l->data);
-        }
-        g_slist_free (manager->priv->windows);
-
         for (i = 0; i < n_screens; i++) {
-                gs_manager_create_window (manager, gdk_display_get_screen (display, i));
+                g_signal_connect (gdk_display_get_screen (display, i),
+                                  "monitors-changed",
+                                  G_CALLBACK (on_screen_monitors_changed),
+                                  manager);
+
+                gs_manager_create_windows_for_screen (manager, gdk_display_get_screen (display, i));
         }
 }
 
@@ -1473,7 +1640,7 @@ gs_manager_activate (GSManager *manager)
         }
 
         if (manager->priv->windows == NULL) {
-                gs_manager_create (GS_MANAGER (manager));
+                gs_manager_create_windows (GS_MANAGER (manager));
         }
 
         manager->priv->jobs = g_hash_table_new_full (g_direct_hash,
@@ -1506,8 +1673,6 @@ gs_manager_activate (GSManager *manager)
 static gboolean
 gs_manager_deactivate (GSManager *manager)
 {
-        GSList *l;
-
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
@@ -1524,11 +1689,7 @@ gs_manager_deactivate (GSManager *manager)
 
         manager_stop_jobs (manager);
 
-        for (l = manager->priv->windows; l; l = l->next) {
-                gs_window_destroy (l->data);
-        }
-        g_slist_free (manager->priv->windows);
-        manager->priv->windows = NULL;
+        gs_manager_destroy_windows (manager);
 
         /* reset state */
         manager->priv->active = FALSE;
